/*
 * key-panel.c -- horrible hack to read the touchpanel
 *
 * Copyright (c) 2000 Alessandro Rubini (rubini@gnu.org)
 * Sponsored by Linuxcare Italia Spa.
 *
 * Extensive modifications by Jorj Bauer <jorj@seas.upenn.edu>,
 * sponsored by DejaVu Software, Inc. (http://www.dejavusoftware.com).
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "key-engine.h"

extern TRANSFORMATION_COEFFICIENTS tc;
int pd_fd = -1;

int kpanel_init(char *tpfile, char *tpdev)
{
    FILE *f;

    if (pd_fd >=0) {
	close(pd_fd);
	pd_fd = -1;
    }

    if (access(tpfile, F_OK|R_OK) < 0) {
      fprintf(stderr, "%s: need to calibrate touch panel first\n", prgname);
      return -1;	   
    }
    f = fopen (tpfile, "r");
    if (!f) {
	fprintf(stderr, "%s: %s: %s\n", prgname, tpfile, strerror(errno));
	exit(1);
    }
    if (fscanf(f, "%d %d %d %d %d %d %d",
	       &tc.a, &tc.b, &tc.c, &tc.d, &tc.e, &tc.f, &tc.s) != 7) {
	fprintf(stderr, "%s: %s: wrong file format\n", prgname, tpfile);
	return -1;
    }
    fclose(f);
    pd_fd = open(tpdev, O_RDONLY);
    if (pd_fd < 0) {
	fprintf(stderr, "%s: %s: %s\n", prgname, tpdev, strerror(errno));
	return -1;
    }
    return 0;
}

int kpanel_read(int pd_fd, int *x, int *y, int *z, int *down)
{
  // Make sure there's new data from the touch panel before we pass it back.
  while (PD_Read(x, y, z, down) == 0)
    ;
  return 0;
}

int kpanel_getstroke(int coords[4])
{
    /*
     * this function is very very dirty, it requires exclusive access
     * to the tp
     */
    int x, y, z, down;

    /* the first is uninteresting (only pen-down) */
    kpanel_read(pd_fd, &x, &y, &z, &down);
    /* get the next one, as landing x and y */
    kpanel_read(pd_fd, coords, coords+1, &z, &down);
    x = coords[0]; y = coords[1];

    while (down) {
	kpanel_read(pd_fd, &coords[2], &coords[3], &z, &down);
	/* if this is more than 20 pixels far, discard it */
	if (abs(coords[2]-x)>20 || abs(coords[3]-y)>20) {
	    coords[2]=x; coords[3]=y; continue;
	}
	x = coords[2]; y = coords[3];
    }
    return 0;
}
