#include <asm/types.h>

typedef struct {
  int x, y;
} MWPOINT;

typedef struct
{
	/*
	 * Coefficients for the transformation formulas:
	 *
	 *     m = (ax + by + c) / s
	 *     n = (dx + ey + f) / s
	 *
	 * These formulas will transform a device point (x, y) to a
	 * screen point (m, n) in fractional pixels.  The fraction
	 * is 1 / TRANSFORMATION_UNITS_PER_PIXEL.
	 */

	int a, b, c, d, e, f, s;
} TRANSFORMATION_COEFFICIENTS;

#define TRANSFORMATION_UNITS_PER_PIXEL 4
#define DATACHANGELIMIT 100
#define MWCOORD int
#define MWBUTTON_L 04
#define EPRINTF(x) fprintf(stderr, x)

#define MODE_COPY 0
#define MODE_XOR 1

extern int PD_Read(MWCOORD *px, MWCOORD *py, MWCOORD *pz, int *pb);
/* 1bpp drawing functions. */
void linear1_drawpixel(char *buffer, fbinfo *devinfo, int x, int y, int c);
void linear1_drawhline(char *buffer, fbinfo *devinfo, int x1, int x2, int y,
		       int c, int mode);
void linear1_drawvline(char *buffer, fbinfo *devinfo, int y1, int y2, int x,
		       int c);

/* 2bpp drawing functions. */
void linear2_drawpixel(char *buffer, fbinfo *devinfo, int x, int y, int c);
void linear2_drawhline(char *buffer, fbinfo *devinfo, int x1, int x2, int y,
		       int c, int mode);
void linear2_drawvline(char *buffer, fbinfo *devinfo, int y1, int y2, int x,
		       int c);

/* 4bpp drawing functions. */
void linear4_drawpixel(char *buffer, fbinfo *devinfo, int x, int y, int c);
void linear4_drawhline(char *buffer, fbinfo *devinfo, int x1, int x2, int y,
		       int c, int mode);
void linear4_drawvline(char *buffer, fbinfo *devinfo, int y1, int y2, int x,
		       int c);

/* 8bpp drawing functions. */
void linear8_drawpixel(char *buffer, fbinfo *devinfo, int x, int y, int c);
void linear8_drawhline(char *buffer, fbinfo *devinfo, int x1, int x2, int y,
		       int c, int mode);
void linear8_drawvline(char *buffer, fbinfo *devinfo, int y1, int y2, int x,
		       int c);

/* 16bpp drawing functions. */
void linear16_drawpixel(char *buffer, fbinfo *devinfo, int x, int y, __u16 c);
void linear16_drawhline(char *buffer, fbinfo *devinfo, int x1, int x2, int y,
			__u16 c, int mode);
void linear16_drawvline(char *buffer, fbinfo *devinfo, int y1, int y2, int x,
			__u16 c);

/* 24bpp drawing functions. */
void linear24_drawpixel(char *buffer, fbinfo *devinfo, int x, int y, 
			__u32 c);
void linear24_drawhline(char *buffer, fbinfo *devinfo, int x1, int x2, int y,
		       __u32 c, int mode);
void linear24_drawvline(char *buffer, fbinfo *devinfo, int y1, int y2, int x,
		       __u32 c);

/* 32bpp drawing functions. */
void linear32_drawpixel(char *buffer, fbinfo *devinfo, int x, int y,
			__u32 c);
void linear32_drawhline(char *buffer, fbinfo *devinfo, int x1, int x2, int y,
		       __u32 c, int mode);
void linear32_drawvline(char *buffer, fbinfo *devinfo, int y1, int y2, int x,
		       __u32 c);


