/*
 * This is a simple program to show data packets. It uses timestamps 
 * to find packet boundaries, so it sometimes clusters several packets
 * in one. Use the "-c" option if you know the packets are constant size
 * (this is often true, but doesn't apply to mman, for example).
 *
 * Obvioulsy, the system should not be loaded in order for timestamping to
 * work.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>

#include <linux/types.h>

extern int errno;

#define DATASIZE 12
int main(int argc, char **argv)
{
    int i, fd = 0; /* stdin */
    unsigned char buf[DATASIZE];
    __s16 *buf16 = (__s16 *)buf;

    /*
     * to prevent a misunderstanding, when using stdin, always print this
     * warning message
     */
    fprintf(stderr, "%s: tool to convert TP binary stream to hex\n", argv[0]);
    if (argc>1) {
	fd = open(argv[1],O_RDONLY);
	if (fd < 0) {
	    fprintf(stderr, "%s: %s: %s\n", argv[0], argv[1], strerror(errno));
	    exit(1);
	}
	fprintf(stderr, "\treading input data from %s ...\n",argv[1]);
    } else
      fprintf(stderr, "\treading input data from stdin ...\n");
    setvbuf(stdout, NULL, _IONBF, 0);
    
    while (1) {

        int res;
        
        if (read(fd, &buf, DATASIZE)<=0) {
	    putchar('\n'); exit(0);
	}
#ifdef BASIC_HEX
	for (i=0; i<DATASIZE; i++) {
	    printf("%02x%c", (int)(buf[i]) & 0xff, i==DATASIZE-1 ? '\n' : ' ');
	}
#else
	if (buf16[0] == 0) {
	    printf("\n");
	    continue;
	}
	if (!buf16[1] && !buf16[2])
	    continue;
#ifdef LOWLEVEL
	for (i=1; i<DATASIZE/2; i++) {
	    printf("%6i %04x%c", (int)(buf16[i]), (int)(buf16[i])&0xffff,
		   i == DATASIZE/2-1 ? '\n' : ' ');
	}
#else
	printf("%4i %4i\n",
	       (buf16[3]+1024-buf16[4])/2,
	       (buf16[2]+1024-buf16[1])/2);
#endif

#endif
    }
}
